#if defined vertex
out vec2 coord;
out vec2 lmcoord;

out vec3 position;
out vec3 worldPosition;
out vec3 normal;
out vec3 tangentVector;

out vec4 color;

flat out float mask;
flat out float material;

out mat3 tbn;

#include "/program/gbuffers/common/curve.vsh"
#include "/program/gbuffers/common/displacement.vsh"
#include "/program/gbuffers/common/material.vsh"
#include "/program/gbuffers/common/tbn.vsh"

void main(){
  coord = gl_MultiTexCoord0.xy;
  lmcoord = gl_MultiTexCoord1.xy / 255.0;

  position = transMAD(gl_ModelViewMatrix, gl_Vertex.xyz);
  worldPosition = renderVertexDisplacement(transMAD(gbufferModelViewInverse, position));
  worldPosition = renderEarthCurvature(worldPosition);
  position = transMAD(gbufferModelView, worldPosition);
  normal = gl_NormalMatrix * gl_Normal;

  color = gl_Color;

  mask = 1.0;
  material = mc_Entity.x;
  renderMaterial(material, mask, lmcoord.x);

  tbn = renderTBN(gl_Normal);

  tangentVector = -normalize((position * gl_NormalMatrix) * tbn);

  #if defined hand
    #ifdef MOVING_HAND
      position.y += sin(frameTimeCounter) * 0.03;
      position.z -= cos(frameTimeCounter) * 0.003;
    #endif

  gl_Position = position.xyzz * diagonal4(gl_ProjectionMatrix) + gl_ProjectionMatrix[3];
  gl_Position.xy += jitter * gl_Position.w;
  #else
  gl_Position = position.xyzz * diagonal4(gbufferProjection) + gbufferProjection[3];
  gl_Position.xy += jitter * gl_Position.w;
  #endif
}
#elif defined fragment
/* DRAWBUFFERS:02 */
layout(location = 0) out vec4 color0;
layout(location = 1) out vec4 color2;

in vec2 coord;
in vec2 lmcoord;

in vec3 position;
in vec3 worldPosition;
in vec3 normal;
in vec3 tangentVector;

in vec4 color;

flat in float mask;
flat in float material;

in mat3 tbn;

bool isWater = material == 1006;
bool isIce = material == 1007;
bool isStainedGlass = material == 1008;

mat2 texD = mat2(dFdx(coord), dFdy(coord));

vec4 data0 = texture2DGrad(texture, coord, texD[0], texD[1]);
vec4 data1 = texture2DGrad(normals, coord, texD[0], texD[1]);
vec4 data2 = texture2DGrad(specular, coord, texD[0], texD[1]);

#if defined water
#include "/program/gbuffers/common/waterNormal.glsl"
#endif

#if defined textured
#include "/program/gbuffers/common/parallax.glsl"
#include "/program/gbuffers/common/puddles.glsl"
#endif

void renderAlbedo(inout vec4 color0){
  #if !defined water
  if(data0.a == 0.0) discard;
  #endif

  vec4 albedo = data0 * color;
  #if defined water
       albedo = isWater ? vec4(1.0) : albedo;
  #endif

  #if TEXTURES == 1
  albedo = vec4(vec3(1.0), albedo.a);
  #endif

  color0 = albedo;
}

void renderNormal(inout float color2){
  vec3 normal = data1.xyz * 2.0 - 1.0;
       normal = data1.xyz == vec3(0.0, 0.0, 0.0) ? vec3(0.0, 0.0, 1.0) : normal;

  #if defined water
    if(isWater){
      vec3 wave = worldPosition + cameraPosition;
      #if WATER_PARALLAX == 1 || WATER_PARALLAX == 2
           wave.xz = renderParallaxWater(wave, tangentVector);
      #endif

      normal = renderWaveNormal(wave);

      #if WATER_PARALLAX == 2
      vec3 viewSpace = transMAD(gbufferModelView, wave - cameraPosition);
  		vec3 clipSpace = (diagonal3(gbufferProjection) * viewSpace + gbufferProjection[3].xyz) / -viewSpace.z * 0.5 + 0.5;
  		gl_FragDepth = isEyeInWater == 0 ? clipSpace.z : gl_FragCoord.z;
    } else {
      gl_FragDepth = gl_FragCoord.z;
      #endif
    }
  #endif

  color2 = EncodeNormal(tbn * normal);
}

float renderTorchlight(float lightmap){
  if(lightmap > 0.001){
    vec3 normalTex = mat3(gbufferModelView) * (tbn * (data1.xyz * 2.0 - 1.0));

    vec3 x = normalize(dFdx(position));
    vec3 y = normalize(dFdy(position));
    vec3 sampleCross = cross(x, y);

    float t0 = dFdx(lmcoord.x) * 256.0;
    float t1 = dFdy(lmcoord.x) * 256.0;

    vec3 t = normalize(t0 * x + 0.0005 * sampleCross + t1 * y);

    float torchpow = clamp(dot(normalTex, t), -1.0, 1.0) * lightmap;
          torchpow = abs(torchpow) > 0.0 ? pow(abs(torchpow), 0.75) * sign(torchpow) : torchpow;
          torchpow = length(t) > 0.001 ? pow(lightmap, 1.0 - torchpow) : torchpow;

    return lightmap;
  }
}

void renderLightmap(inout float color2){
  float torchlight = clamp01(renderTorchlight(lmcoord.x) + (dither64(gl_FragCoord.xy) / 128.0));
  float lightmap = clamp01(lmcoord.y + (dither64(gl_FragCoord.xy) / 128.0));

  color2 = EncodeVec2(torchlight, lightmap);
}

float rendereOldf0(float metalness) {
	float f0 = 1.0;

	switch(int(material)) {
		case 1018:
		case 1016: f0 = 0.97; break; // Gold
		case 1015: f0 = 0.46; break; // Iron
		case 1017: f0 = 0.35; break; // Diamond
		default: break;
	}

	return f0 * metalness;
}

void renderSpecular(inout float color2){
  vec4 spec = data2;

  #if SPEC_FORMAT == 0
  float roughness = 1.0 - spec.z;
  float f0 = spec.x * 0.2;
  #else
  float roughness = 1.0 - spec.x;
  float f0 = rendereOldf0(spec.y);
  #endif

  #if defined textured
    #ifdef PUDDLES
      roughness = mix(roughness, 0.03, renderPuddlesRain(worldPosition + cameraPosition));
      f0 = mix(f0, 0.021, renderPuddlesRain(worldPosition + cameraPosition));
    #endif
  #endif

  #if defined water
    #ifdef STAINED_GLASS_REFLECTION
      roughness = isWater || isIce || isStainedGlass ? 0.03 : roughness;
      f0 = isWater || isIce || isStainedGlass ? 0.021 : f0;
    #else
      roughness = isWater || isIce ? 0.03 : roughness;
      f0 = isWater || isIce ? 0.021 : f0;
    #endif
  #endif

  color2 = EncodeVec2(roughness, f0);
}

void main(){
  #if defined water
  	if(!gl_FrontFacing) discard;
  #endif

  renderAlbedo(color0);

  renderNormal(color2.x);
  renderLightmap(color2.y);
  renderSpecular(color2.z);

  #if defined entities && defined basic
  color2.w = EncodeVec2(1.0, 1.0);
  #else
  color2.w = EncodeVec2(1.0 - mask, 1.0);
  #endif
}
#endif
