//PBR volumetric Clouds modified version from robobo1221
//https://www.shadertoy.com/view/MstBWs
//https://www.shadertoy.com/view/WdsSzr

float phase2Lobes(float x){
  const float m = 0.60;
  const float gm = 0.80;

	float lobe1 = phase_mie(x, 0.8 * gm);
  float lobe2 = phase_mie(x, -0.5 * gm);

  return mix(lobe2, lobe1, m);
}

float renderScatterIntergral(float opticalDepth, float coeff){
  float a = -coeff * rLOG2;
  float b = -1.0 / coeff;
  float c =  1.0 / coeff;

  return exp2(a * opticalDepth) * b + c;
}

float remap(float value, const float originalMin, const float originalMax, const float newMin, const float newMax) {
	return (((value - originalMin) / (originalMax - originalMin)) * (newMax - newMin)) + newMin;
}

float renderVolumeFBM(vec3 p){

	p.y -= dot(p.xz, p.xz) / earthSize;

	if (p.y < cloudMinHeight || p.y > cloudMaxHeight) return 0.0;

  float time = TIME * 0.02;
  vec3 movement = vec3(time, 0.0, time);

  vec3 cloudCoord = (p * 0.0025) * scale - movement;

  float noise = renderFBM(cloudCoord, movement);

	float coverage = (VOLUME_CLOUDS_COVERAGE) + (wetness * VOLUME_CLOUDS_COVERAGE_WEATHER);

	float horizonHeight = p.y - cloudMinHeight;
	float scaleHeight = horizonHeight / thickness;

	float heighAtten = remap(scaleHeight, 0.0, 0.4, 0.0, 1.0) * remap(scaleHeight, 0.6, 1.0, 1.0, 0.0);

  return max0(noise * coverage - (0.6 * heighAtten * scaleHeight * 0.5 + 0.3)) * (density * scale);
}

float renderSunVisibility(vec3 p){
	const int steps = 5;
	const float rSteps = thickness / steps;

	vec3 increment = worldLightVector * rSteps;
	float rayLength = length(increment);

	float opticalDepth = 0.0;

	for(int i = 0; i < steps; i++, p += increment){
		opticalDepth += renderVolumeFBM(p);
	}

	return exp2(-opticalDepth * rSteps * rLOG2);
}

float renderCloudTransmittance(float bn, float transmittanceDepth){
  return exp2(-transmittanceDepth * bn);
}

float renderPowder(float od){
  return 1.0 - exp2(-od * 2.0);
}

vec3 renderVolumeCloudsScattering(float opticalDepth, float sampleTransmittance, float VdotL, vec3 p){
	float transmittance = renderScatterIntergral(sampleTransmittance, 1.11);
  float powder = renderPowder(opticalDepth);

  vec3 sunlighting = vec3(0.0);
  vec3 skylighting = vec3(0.0);

	float direct = renderSunVisibility(p);

	for(int i = 0; i < 5; ++i){
		float n = float(i);

		float an = pow(0.15, n);
		float bn = pow(0.5, n);
		float cn = pow(0.85, n);

		transmittance = transmittance * an;
		float phase = phase2Lobes(VdotL * cn);

		sunlighting += (sunlight + moonlight) * renderCloudTransmittance(bn, direct) * phase * direct * powder * transmittance * transitionFading * 3.0;
		skylighting += FromSH(skySH[0], skySH[1], skySH[2], vec3(0.0, 1.0, 0.0)) * 0.01 * transmittance * rPI;
	}

  return (sunlighting + skylighting) * PI;
}

vec3 renderVolumeClouds(vec3 color, vec3 worldPosition, vec3 worldVector, float VdotL, float depth, float dither, const int steps){
#ifdef VOLUME_CLOUDS
  const float rSteps = 1.0 / steps;

  dither = fract(frameTimeCounter / 16.0 + dither);

  vec3 start = worldPosition * (cloudMinHeight - cameraPosition.y) / worldPosition.y;
       start = mix(start, vec3(0.0), cameraPosition.y >= cloudMinHeight && cameraPosition.y <= cloudMaxHeight);
       start = mix(start, worldPosition * (cloudMaxHeight - cameraPosition.y) / worldPosition.y, cameraPosition.y >= cloudMaxHeight);
	     start = mix(gbufferModelViewInverse[3].xyz, start, depth >= 1.0);

  vec3 increment = worldVector * altitude * rSteps / clamp01(abs(worldVector.y));
       increment = mix((worldPosition - start) * rSteps, increment, depth >= 1.0);

  vec3 position  = increment * dither + start + cameraPosition;
  float rayLength = length(increment);

  float transmittance = 1.0;
  vec3 scattering = vec3(0.0);

  for (int i = 0; i < steps; ++i, position += increment) {
		float opticalDepth = renderVolumeFBM(position) * rayLength;
    if (opticalDepth <= 0.0) continue;

    float sampleTransmittance = exp2(-opticalDepth * 1.11);

		scattering += renderVolumeCloudsScattering(opticalDepth, sampleTransmittance, VdotL, position) * (-transmittance * sampleTransmittance + transmittance);
		transmittance *= sampleTransmittance;
  }

  return mix(color * transmittance + scattering, color, clamp01(length(start) * 0.0001));
#else
  return color;
#endif
}

float renderCloudShadow(vec3 position, vec3 direction){
  #ifndef VOLUME_CLOUDS
  return 1.0;
  #endif

  const int steps = CLOUD_SHADOW_QUALITY;
  const float rSteps = thickness / steps;

	float fade = smoothstep(0.125, 0.075, abs(direction.y));

	float stepLength = rSteps / direction.y;
  vec3 increment = direction * stepLength;

  position += position.y <= cloudMinHeight ? direction * (cloudMinHeight - position.y) / direction.y : vec3(0.0);

  float transmittance = 0.0;

  for (int i = 0; i < steps; ++i, position += increment){
		transmittance += renderVolumeFBM(position) ;
  }

  return exp2(-transmittance * stepLength * rLOG2) * (1.0 - fade) + fade;
}
