//PBR volumetric Ray from robobo1221
//https://www.shadertoy.com/view/MstBWs

vec3 renderScatterIntergral(float opticalDepth, vec3 coeff){
  vec3 a = -coeff * rLOG2;
  vec3 b = -1.0 / coeff;
  vec3 c =  1.0 / coeff;

  return exp2(a * opticalDepth) * b + c;
}

vec3 renderScatterIntergral(vec3 stepTransmittance, const vec3 coeff){
  const vec3 a = -1.0 / coeff;

  return stepTransmittance * a - a;
}

float renderVolumeFog(vec3 position, const float level){
  const  float fogThickness = 500.0;

  float time = TIME * 0.15;
  vec3 movement = vec3(time, 0.0, time);

	vec3 p = (position * 0.00045) * fogThickness - movement;

  float noise = renderFBM(p, movement);

  float height = position.y - level;
  float scaleHeight = height / fogThickness;
  float heighAtten = remap(scaleHeight, 0.0, 0.4, 0.0, 1.0) * remap(scaleHeight, 0.6, 1.0, 1.0, 0.0);

  float fog = max0(noise - (0.6 * heighAtten * scaleHeight * 0.5 + 0.35));
      	fog *= exp2(-max0(height) * 0.3);

  return fog * VOLUME_FOG_DENSITY * (1.0 - wetness * 0.8);
}

vec2 renderOpticalDepth(vec3 position){
  const float level = 63.0;
  const float scale = 20.0;

  vec2 opticalDepth = exp2(-max0(position.y - level) * rScaleHeigh * rLOG2);
       opticalDepth += exp2(-max0(position.y - level) * rScaleHeigh * vec2(scale) * rLOG2) * scale;
       opticalDepth.y += exp2(-max0(position.y - level) * 0.02 * rLOG2) * 350.0;

  #ifdef VOLUME_FOG
       opticalDepth += renderVolumeFog(position, level) * vec2(500.0, 2500.0) * renderPowder(opticalDepth.y) * 3e1 * sunRise;
  #endif

  return opticalDepth * (1.0 - wetness * 0.8);
}

vec3 renderVolumetricRayShadow(vec3 position, vec3 shadowPosition, bool water){
  const float lod = 2;

  vec3 shadow = vec3(0.0);

  shadowPosition = renderShadowDistortPosition(shadowPosition) * 0.5 + 0.5;

  vec2 depth = vec2(texture2DLod(shadowtex0, shadowPosition.xy, lod).x, texture2DLod(shadowtex1, shadowPosition.xy, lod).x);

  vec4 color0 = texture2DLod(shadowcolor0, shadowPosition.xy, lod);
  vec4 color1 = texture2DLod(shadowcolor1, shadowPosition.xy, lod);

  float shadow0 = step(shadowPosition.z, depth.x);
  float shadow1 = step(shadowPosition.z, depth.y);

  #ifdef COLOR_SHADOW
    shadow += mix(vec3(shadow0), toGamma(color0.xyz), clamp01(shadow1 - shadow0));
  #else
    shadow += shadow1;
  #endif

  if(!water) return shadow;

  float waterDepth = (depth.x * 8.0 - 4.0) * shadowProjectionInverse[2].z + shadowProjectionInverse[3].z;
        waterDepth = (waterDepth - transMAD(shadowModelView, position).z);
  if(waterDepth <= 0.0) return shadow;
       waterDepth = mix(0.0, waterDepth, color0.a);

  vec3 waterTransmittance = exp2(-waterTransmittanceCoefficient * waterDepth);

  #ifdef COLOR_SHADOW
    shadow += mix(vec3(shadow0), color0.xyz, clamp01(shadow1 - shadow0)) * waterTransmittance;
  #else
    shadow += shadow1 * waterTransmittance;
  #endif

  return shadow;
}

vec3 renderVolumetricRay(vec3 color, vec3 start, vec3 end, DataBuffer data, Material material){
  #ifndef VOLUMETRIC_RAY
  return color;
  #endif

  #if !defined world0
  return color;
  #endif

  const int steps = VOLUMETRIC_RAY_QUALITY;
  const float rSteps = 1.0 / steps;

  #ifdef TEMPORAL_AA
  data.dither += fract(frameTimeCounter / 16.0 + data.dither);
  #endif

  vec3 increment = (end - start) * rSteps;
  vec3 position = increment * data.dither + start;

  vec3 shadowStart = transMAD(shadowMatrix, start);
  vec3 shadowEnd = transMAD(shadowMatrix, end);

  vec3 shadowIncrement = (shadowEnd - shadowStart) * rSteps;
  vec3 shadowPosition = shadowIncrement * data.dither + shadowStart;

  float VdotS = dot(data.vector[1], lightVector);
  vec2 phase = phase_Sky(VdotS, sky_mieg);

  vec3 scattering = vec3(0.0);
  vec3 transmittance = vec3(1.0);

  for(int i = 0; i < steps; ++i, position += increment, shadowPosition += shadowIncrement){
    vec2 opticalDepth = renderOpticalDepth(position + cameraPosition) * length(increment);

		mat2x3 scatter = mat2x3(scatteringCoeff[0] * renderScatterIntergral(opticalDepth.x, attenuationCoeff[0]),
														scatteringCoeff[1] * renderScatterIntergral(opticalDepth.y, attenuationCoeff[1]));

    vec3 shadow = renderVolumetricRayShadow(position, shadowPosition, false);
    #ifdef CLOUD_SHADOW
         //shadow *= renderCloudShadow(position + cameraPosition, worldLightVector);
    #endif

    vec3 direct = (sunlight + moonlight) * (scatter * phase) * shadow * transitionFading;
    vec3 indirect = FromSH(skySH[0], skySH[1], skySH[2], worldUpVector) * (scatter * vec2(0.20)) * ambientFog;

    scattering += (direct + indirect) * transmittance;
    transmittance *= exp2(-mat2x3(attenuationCoeff) * (opticalDepth * rLOG2));
  }

  return color * transmittance + scattering;
}

vec3 renderVolumetricRayWater(vec3 color, vec3 start, vec3 end, DataBuffer data, Material material){
  #ifndef VOLUMETRIC_RAY
  return color;
  #endif

  #if !defined world0
  return color;
  #endif

  const int steps = VOLUMETRIC_RAY_QUALITY;
  const float rSteps = 1.0 / steps;

  #ifdef TEMPORAL_AA
  data.dither += fract(frameTimeCounter / 16.0 + data.dither);
  #endif

  vec3 increment = (end - start) * rSteps;
  vec3 position = increment * data.dither + start;

  vec3 shadowStart = transMAD(shadowMatrix, start);
  vec3 shadowEnd = transMAD(shadowMatrix, end);

  vec3 shadowIncrement = (shadowEnd - shadowStart) * rSteps;
  vec3 shadowPosition = shadowIncrement * data.dither + shadowStart;

  float VdotS = dot(data.vector[1], lightVector);
  float phase = phase_mie(VdotS, sky_mieg);

  vec3 scattering = vec3(0.0);
  vec3 transmittance = vec3(1.0);

  for(int i = 0; i < steps; ++i, position += increment, shadowPosition += shadowIncrement){
    vec3 opticalDepth = exp2(-waterAbsorptionCoefficient * length(increment));
    vec3 scatter = waterScatterCoefficient * renderScatterIntergral(opticalDepth, waterTransmittanceCoefficient);

    vec3 shadow = renderVolumetricRayShadow(position, shadowPosition, true);
    #ifdef CLOUD_SHADOW
         //shadow *= renderCloudShadow(position + cameraPosition, worldLightVector);
    #endif

    vec3 direct = (sunlight + moonlight) * shadow * opticalDepth * phase * transitionFading;
    vec3 indirect = FromSH(skySH[0], skySH[1], skySH[2], worldUpVector) * 0.20 * opticalDepth * hPI;

    scattering += (direct + indirect) * scatter;
    transmittance *= opticalDepth;
  }

  return max0(color * transmittance + scattering);
}
